@@ -49,19 +49,19 @@ typedef struct {
 
 
 
-static int process_message(idmef_message_t *msg, void *priv) 
+static int process_message(idmef_message_t *msg, void *priv)
 {
-	filter_plugin_t *plugin = priv;
-	int ret;
-        
+        filter_plugin_t *plugin = priv;
+        int ret;
+
         if ( ! plugin->criteria )
                 return 0;
 
         ret = idmef_criteria_match(plugin->criteria, msg);
         if ( ret < 0 )
                 prelude_perror(ret, "error matching criteria");
-        
-	return (ret > 0) ? 0 : -1;
+
+        return (ret > 0) ? 0 : -1;
 }
 
 
@@ -69,7 +69,7 @@ static int get_filter_hook(prelude_option_t *opt, prelude_string_t *out, void *c
 {
         int ret = 0;
         filter_plugin_t *plugin;
-        
+
         plugin = prelude_plugin_instance_get_plugin_data(context);
 
         if ( plugin->hook_str )
@@ -80,7 +80,7 @@ static int get_filter_hook(prelude_option_t *opt, prelude_string_t *out, void *c
 
 
 
-static int set_filter_hook(prelude_option_t *opt, const char *optarg, prelude_string_t *err, void *context) 
+static int set_filter_hook(prelude_option_t *opt, const char *optarg, prelude_string_t *err, void *context)
 {
         int i, ret;
         filter_plugin_t *plugin;
@@ -96,10 +96,10 @@ static int set_filter_hook(prelude_option_t *opt, const char *optarg, prelude_st
         };
 
         plugin = prelude_plugin_instance_get_plugin_data(context);
-        
+
         for ( i = 0; tbl[i].hook != NULL; i++ ) {
                 ret = strcasecmp(optarg, tbl[i].hook);
-                if ( ret == 0 ) {                        
+                if ( ret == 0 ) {
                         manager_filter_new_hook(&plugin->hook, context, tbl[i].cat, NULL, plugin);
                         goto success;
                 }
@@ -110,7 +110,7 @@ static int set_filter_hook(prelude_option_t *opt, const char *optarg, prelude_st
                 prelude_string_sprintf(err, "error parsing value: '%s'", optarg);
                 return -1;
         }
-        
+
         ptr = prelude_plugin_search_instance_by_name(NULL, pname, (ret == 2) ? iname : NULL);
         if ( ! ptr ) {
                 prelude_string_sprintf(err, "Unknown hook '%s'", optarg);
@@ -122,11 +122,11 @@ static int set_filter_hook(prelude_option_t *opt, const char *optarg, prelude_st
  success:
         if ( plugin->hook_str )
                 free(plugin->hook_str);
-        
+
         plugin->hook_str = strdup(optarg);
         if ( ! plugin->hook_str )
                 return -1;
-        
+
         return 0;
 }
 
@@ -136,14 +136,14 @@ static int add_criteria(filter_plugin_t *plugin, const char *criteria)
 {
         int ret;
         idmef_criteria_t *new;
-        
+
         ret = idmef_criteria_new_from_string(&new, criteria);
-        if ( ret < 0 ) 
+        if ( ret < 0 )
                 return ret;
 
         if ( plugin->criteria )
                 idmef_criteria_destroy(plugin->criteria);
-        
+
         plugin->criteria = new;
 
         return 0;
@@ -158,27 +158,22 @@ static int read_criteria_from_filename(filter_plugin_t *plugin, const char *file
         prelude_string_t *out;
         unsigned int line = 0;
         idmef_criteria_t *new, *criteria = NULL;
-        
+
         fd = fopen(filename, "r");
         if ( ! fd ) {
                 prelude_string_sprintf(err, "error opening '%s' for reading: %s (%d)", filename, strerror(errno), errno);
                 return -1;
         }
-        
+
         ret = prelude_string_new(&out);
         if ( ret < 0 )
                 return ret;
-        
-        while ( prelude_read_multiline2(fd, &line, out) == 0 ) {
-                
+
+        while ( (ret = prelude_read_multiline2(fd, &line, out)) == 0 ) {
                 ret = idmef_criteria_new_from_string(&new, prelude_string_get_string(out));
                 if ( ret < 0 ) {
                         prelude_string_sprintf(err, "%s:%u: %s", filename, line, prelude_strerror(ret));
-
-                        if ( criteria ) idmef_criteria_destroy(criteria);
-                        criteria = NULL;
-
-                        break;
+                        goto err;
                 }
 
                 if ( criteria )
@@ -186,7 +181,15 @@ static int read_criteria_from_filename(filter_plugin_t *plugin, const char *file
                 else
                         criteria = new;
         }
-        
+
+        if ( ret < 0 && prelude_error_get_code(ret) != PRELUDE_ERROR_EOF ) {
+                prelude_string_sprintf(err, "error reading '%s': %s", filename, prelude_strerror(ret));
+                return ret;
+        }
+
+        ret = 0;
+
+err:
         prelude_string_destroy(out);
         fclose(fd);
 
@@ -200,9 +203,9 @@ static int read_criteria_from_filename(filter_plugin_t *plugin, const char *file
 
 
 
-static int set_filter_rule(prelude_option_t *opt, const char *optarg, prelude_string_t *err, void *context) 
+static int set_filter_rule(prelude_option_t *opt, const char *optarg, prelude_string_t *err, void *context)
 {
-	int ret;
+        int ret;
         filter_plugin_t *plugin = prelude_plugin_instance_get_plugin_data(context);
 
         ret = access(optarg, R_OK);
@@ -217,24 +220,24 @@ static int set_filter_rule(prelude_option_t *opt, const char *optarg, prelude_st
 
 static int get_filter_rule(prelude_option_t *opt, prelude_string_t *out, void *context)
 {
-        filter_plugin_t *plugin = prelude_plugin_instance_get_plugin_data(context);       
+        filter_plugin_t *plugin = prelude_plugin_instance_get_plugin_data(context);
         return idmef_criteria_to_string(plugin->criteria, out);
 }
 
 
 
 
-static int filter_activate(prelude_option_t *opt, const char *optarg, prelude_string_t *err, void *context) 
+static int filter_activate(prelude_option_t *opt, const char *optarg, prelude_string_t *err, void *context)
 {
         filter_plugin_t *new;
-        
+
         new = calloc(1, sizeof(*new));
         if ( ! new )
                 return prelude_error_from_errno(errno);
-        
+
         new->criteria = NULL;
         prelude_plugin_instance_set_plugin_data(context, new);
-        
+
         return 0;
 }
 
@@ -252,7 +255,7 @@ static void filter_destroy(prelude_plugin_instance_t *pi, prelude_string_t *out)
 
         if ( plugin->hook_str )
                 free(plugin->hook_str);
-        
+
         free(plugin);
 }
 
@@ -263,7 +266,7 @@ int idmef_criteria_LTX_manager_plugin_init(prelude_plugin_entry_t *pe, void *roo
 {
         int ret;
         prelude_option_t *opt;
-        
+
         ret = prelude_option_add(root_opt, &opt, PRELUDE_OPTION_TYPE_CLI|PRELUDE_OPTION_TYPE_CFG
                                  |PRELUDE_OPTION_TYPE_WIDE, 0, "idmef-criteria",
                                  "Filter message based on IDMEF criteria", PRELUDE_OPTION_ARGUMENT_OPTIONAL,
@@ -273,21 +276,21 @@ int idmef_criteria_LTX_manager_plugin_init(prelude_plugin_entry_t *pe, void *roo
 
         prelude_option_set_priority(opt, PRELUDE_OPTION_PRIORITY_LAST);
         prelude_plugin_set_activation_option(pe, opt, NULL);
-        
+
         ret = prelude_option_add(opt, NULL, PRELUDE_OPTION_TYPE_CLI|PRELUDE_OPTION_TYPE_CFG
                                  |PRELUDE_OPTION_TYPE_WIDE, 'r', "rule",
                                  "Filter rule, or filename containing rule", PRELUDE_OPTION_ARGUMENT_REQUIRED,
                                  set_filter_rule, get_filter_rule);
         if ( ret < 0 )
                 return ret;
-        
+
         ret = prelude_option_add(opt, NULL, PRELUDE_OPTION_TYPE_CLI|PRELUDE_OPTION_TYPE_CFG
                                  |PRELUDE_OPTION_TYPE_WIDE, 0, "hook",
                                  "Where the filter should be hooked (reporting|reverse-relaying|plugin name)",
                                  PRELUDE_OPTION_ARGUMENT_REQUIRED, set_filter_hook, get_filter_hook);
         if ( ret < 0 )
                 return ret;
-        
+
         prelude_plugin_set_name(&filter_plugin, "IDMEF-Criteria");
         prelude_plugin_set_destroy_func(&filter_plugin, filter_destroy);
         manager_filter_plugin_set_running_func(&filter_plugin, process_message);
